//
//  LASAnalytics.h
//  LASAnalytics
//
//  Created by Jean-Denis Rocheteau on 2014-07-27.
//  Copyright (c) 2014 LightSpeed retail, inc. All rights reserved.
//

#import <Foundation/Foundation.h>

#if TARGET_OS_IPHONE
#import <LASAnalytics/UIViewController+Analytics.h>
#endif

NS_ASSUME_NONNULL_BEGIN

@interface LASAnalytics : NSObject

/**
 * Default traits that will be sent with identify tracking events
 */
@property (copy) NSDictionary *defaultIdentifyTraits;

/**
 * Default properties that will be sent with screen tracking events
 */
@property (copy) NSDictionary *defaultScreenProperties;

/**
 * Default properties that will be sent with track tracking events
 */
@property (copy) NSDictionary *defaultTrackProperties;

/**
 * NSSet containing all the "screen" events sent to the analytics service
 */
@property (copy) NSSet* screenWhitelist;

/**
 * NSSet containing all the "track" events sent to the analytics service
 */
@property (copy) NSSet* trackWhitelist;

/**
 * iOS device token, used for APNS
 */
@property (copy, nonatomic) NSData *deviceToken;

/**
 * App Name that is encoded in the title of some analytics events. This is loaded automatically from info.plist for the app bundle,
 * using the "LASAnalytics Configuration/Analytics App Prefix" keyPath.  Read-write for dependency injection for testing.
 */
@property (nullable, copy) NSString *appNameForEventTitles;

/**
 * Shared instance. You must make sure setupAnalyticsWithKey:standardTraits: is called
 * before starting to use the shared instance.
 *
 *  @return shared analytics instance
 */
+ (LASAnalytics*)sharedAnalytics;

/**
 *  This call must be made as soon as possible to tie one the current user to its actions
 *
 *  @param userId The rad ID for this user (customer account).
 */
- (void)identify:(NSString *)userId;

/**
 *  This call must be made as soon as possible to tie one the current user to its actions
 *
 *  @param userId The rad ID for this user (customer account).
 *  @param traits A dictionary of traits you know about the user, like the main email address or the number of shops for this account.
 */
- (void)identify:(NSString *)userId traits:(nullable NSDictionary *)traits;

/**
 *  Clear the SDK’s internal data. This is useful for apps where users can log in and out with different identities over time.
 */
- (void)reset;

/**
 *  Record the app screens the users view, along with optional extra information about the screen being viewed.
 *
 *  @param screenTitle The name of the screen.
 */
- (void)screen:(NSString *)screenTitle;

/**
 *  Record the app screens the users view, along with optional extra information about the screen being viewed.
 *
 *  @param screenTitle The name of the screen.
 *  @param properties  A dictionary of properties for the event.
 */
- (void)screen:(NSString *)screenTitle properties:(nullable NSDictionary *)properties;

/**
 *  Configure analytics with project token and standard traits to transfer in every API call.
 *  @param token Project token
 */
- (void)setUpAnalyticsWithToken:(NSString*)token;

/**
 *  Send an action event for the current user.
 *
 *  @param event      The name of the event.
 */
- (void)track:(NSString *)event;

/**
 *  Send an action event for the current user.
 *
 *  @param event      The name of the event.
 *  @param properties A dictionary of properties for the event. If the event was 'Added to Cart', it might have properties like price and productType.
 */
- (void)track:(NSString *)event properties:(nullable NSDictionary *)properties;

/**
 *  Send an action event triggered by a hardware device
 *
 *  @param hardwareType descriptor for the type of hardware (e.g. "Barcode scanner")
 */
- (void)trackFromHardwareType:(NSString*)hardwareType;

/**
 *  Send an action event triggered by a hardware device
 *
 *  @param hardwareType descriptor for the type of hardware (e.g. "Barcode scanner")
 *  @param properties   A dictionary of properties for the hardware event (e.g. firmware version, model, etc.)
 */
- (void)trackFromHardwareType:(NSString*)hardwareType properties:(nullable NSDictionary *)properties;

@end

NS_ASSUME_NONNULL_END

//! Project version number for LASAnalytics.
FOUNDATION_EXPORT double LASAnalyticsVersionNumber;

//! Project version string for LASAnalytics.
FOUNDATION_EXPORT const unsigned char LASAnalyticsVersionString[];
